@@ -1,4 +1,4 @@
-// Copyright 2018 MaidSafe.net limited.
+// Copyright 2021 MaidSafe.net limited.
 //
 // This SAFE Network Software is licensed to you under The General Public License (GPL), version 3.
 // Unless required by applicable law or agreed to in writing, the SAFE Network Software distributed
@@ -8,18 +8,17 @@
 
 #![doc(hidden)]
 
-use super::Storage;
-use crate::SelfEncryptionError;
-use async_trait::async_trait;
+use crate::Error;
 
-use rand::{self, Rng, SeedableRng};
+use bytes::Bytes;
+use rand::{self, rngs::OsRng, Rng, SeedableRng};
 use rand_chacha::ChaChaRng;
+use rayon::current_num_threads;
 use serde::{de::DeserializeOwned, Serialize};
 use std::{
     cmp, env,
     fmt::{self, Debug, Formatter},
     sync::{Arc, RwLock},
-    thread,
 };
 use tiny_keccak::{Hasher, Sha3};
 
@@ -43,8 +42,8 @@ impl<'a> Debug for Blob<'a> {
 
 #[derive(Clone)]
 struct Entry {
-    name: Vec<u8>,
-    data: Vec<u8>,
+    name: Bytes,
+    data: Bytes,
 }
 
 #[derive(Default, Clone)]
@@ -59,109 +58,124 @@ impl SimpleStorage {
         }
     }
 
-    pub async fn has_chunk(&self, name: &[u8]) -> Result<bool, SelfEncryptionError> {
+    pub async fn has_chunk(&self, name: &[u8]) -> Result<bool, Error> {
         Ok(self
             .entries
             .read()
-            .map_err(|_| SelfEncryptionError::Poison)?
+            .map_err(|_| Error::Poison)?
             .iter()
             .any(|entry| entry.name == name))
     }
 
-    pub async fn num_entries(&self) -> Result<usize, SelfEncryptionError> {
-        Ok(self
-            .entries
-            .read()
-            .map_err(|_| SelfEncryptionError::Poison)?
-            .len())
+    pub async fn num_entries(&self) -> Result<usize, Error> {
+        Ok(self.entries.read().map_err(|_| Error::Poison)?.len())
     }
-}
 
-#[async_trait]
-impl Storage for SimpleStorage {
-    // type Error = SelfEncryptionError;
-
-    async fn get(&mut self, name: &[u8]) -> Result<Vec<u8>, SelfEncryptionError> {
+    pub async fn get(&self, name: Bytes) -> Result<Bytes, Error> {
         match self
             .entries
             .read()
-            .map_err(|_| SelfEncryptionError::Poison)?
+            .map_err(|_| Error::Poison)?
             .iter()
             .find(|entry| entry.name == name)
         {
             Some(entry) => Ok(entry.data.clone()),
-            None => Err(SelfEncryptionError::Storage(
-                "Chunk missing in storage".into(),
-            )),
+            None => Err(Error::Storage("Chunk missing in storage".into())),
         }
     }
 
-    async fn put(&mut self, name: Vec<u8>, data: Vec<u8>) -> Result<(), SelfEncryptionError> {
+    pub async fn put(&self, name: Bytes, data: Bytes) -> Result<(), Error> {
         self.entries
             .write()
-            .map_err(|_| SelfEncryptionError::Poison)?
+            .map_err(|_| Error::Poison)?
             .push(Entry { name, data });
 
         Ok(())
     }
 
-    async fn delete(&mut self, name: &[u8]) -> Result<(), SelfEncryptionError> {
+    pub async fn delete(&self, name: Bytes) -> Result<(), Error> {
         self.entries
             .write()
-            .map_err(|_| SelfEncryptionError::Poison)?
+            .map_err(|_| Error::Poison)?
             .retain(|entry| entry.name != name);
 
         Ok(())
     }
 
-    async fn generate_address(&self, data: &[u8]) -> Result<Vec<u8>, SelfEncryptionError> {
+    pub async fn generate_address(&self, data: Bytes) -> Result<Bytes, Error> {
         let mut hasher = Sha3::v256();
         let mut output = [0; 32];
-        hasher.update(&data);
+        hasher.update(data.as_ref());
         hasher.finalize(&mut output);
-        Ok(output.to_vec())
+        Ok(Bytes::from(output.to_vec()))
     }
 }
 
 // Create new random number generator suitable for tests. To provide repeatable results, the seed
 // can be overridden using the "SEED" env variable. If this variable is not provided, a random one
 // is used (to support soak testing). The current seed is printed to stdout.
-pub fn new_test_rng() -> Result<TestRng, SelfEncryptionError> {
+pub fn new_test_rng() -> Result<TestRng, Error> {
     let seed = if let Ok(seed) = env::var("SEED") {
         seed.parse()?
     } else {
         rand::thread_rng().gen()
     };
 
-    println!(
-        "RNG seed for thread {:?}: {}",
-        thread::current().name().unwrap(),
-        seed
-    );
+    // println!(
+    //     "RNG seed for thread {:?}: {}",
+    //     thread::current().name().unwrap(),
+    //     seed
+    // );
 
     Ok(TestRng::seed_from_u64(seed))
 }
 
-pub fn from_rng(rng: &mut TestRng) -> Result<TestRng, SelfEncryptionError> {
+pub fn from_rng(rng: &mut TestRng) -> Result<TestRng, Error> {
     Ok(TestRng::from_rng(rng)?)
 }
 
-pub fn serialise<T: Serialize>(data: &T) -> Result<Vec<u8>, SelfEncryptionError> {
+pub fn serialise<T: Serialize>(data: &T) -> Result<Vec<u8>, Error> {
     Ok(bincode::serialize(data)?)
 }
 
-pub fn deserialise<T>(data: &[u8]) -> Result<T, SelfEncryptionError>
+pub fn deserialise<T>(data: &[u8]) -> Result<T, Error>
 where
     T: Serialize + DeserializeOwned,
 {
     match bincode::deserialize(data) {
         Ok(data) => Ok(data),
-        Err(_) => Err(SelfEncryptionError::Deserialise),
+        Err(_) => Err(Error::Deserialise),
     }
 }
 
-pub fn random_bytes<T: Rng>(rng: &mut T, size: usize) -> Vec<u8> {
-    let mut bytes = vec![0_u8; size];
-    rng.fill(bytes.as_mut_slice());
-    bytes
+/// Generates random bytes using provided `size`.
+pub fn random_bytes(size: usize) -> Bytes {
+    use rayon::prelude::*;
+    let threads = current_num_threads();
+
+    if threads > size {
+        let mut rng = OsRng;
+        return ::std::iter::repeat(())
+            .map(|()| rng.gen::<u8>())
+            .take(size)
+            .collect();
+    }
+
+    let per_thread = size / threads;
+    let remainder = size % threads;
+
+    let mut bytes: Vec<u8> = (0..threads)
+        .par_bridge()
+        .map(|_| vec![0u8; per_thread])
+        .map(|mut bytes| {
+            let bytes = bytes.as_mut_slice();
+            rand::thread_rng().fill(bytes);
+            bytes.to_owned()
+        })
+        .flatten()
+        .collect();
+
+    bytes.extend(vec![0u8; remainder]);
+
+    Bytes::from(bytes)
 }
